var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
import OAuthSimple from "oauthsimple";
var OAuth = /** @class */ (function () {
    function OAuth(provider, consumer, store) {
        if (store === void 0) { store = window.localStorage; }
        this.provider = provider;
        this.consumer = consumer;
        this.store = store;
        this.__tempOAuthTokenSecret = null;
    }
    OAuth.prototype.getProviderAuthorizeURL = function (callback) {
        return __awaiter(this, void 0, void 0, function () {
            var token, authorizepage;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getOneTimeRequestToken(callback)];
                    case 1:
                        token = (_a.sent()).token;
                        authorizepage = new URL(this.provider.authorize_url);
                        authorizepage.searchParams.set("oauth_token", token);
                        return [2 /*return*/, authorizepage];
                }
            });
        });
    };
    OAuth.prototype.getOneTimeRequestToken = function (callback) {
        return __awaiter(this, void 0, void 0, function () {
            var request, res, params, _a;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        request = OAuthSimple().sign({
                            path: this.provider.request_url,
                            parameters: {
                                xoauth_displayname: this.consumer.app_name,
                                scope: this.consumer.scope || "write",
                                oauth_callback: callback,
                            },
                            signatures: {
                                consumer_key: this.consumer.consumer_key,
                                shared_secret: this.consumer.consumer_secret,
                            },
                        });
                        return [4 /*yield*/, fetch(request.signed_url, {
                                method: "GET",
                                headers: { "Authorization": request.header },
                            })];
                    case 1:
                        res = _b.sent();
                        _a = URLSearchParams.bind;
                        return [4 /*yield*/, res.text()];
                    case 2:
                        params = new (_a.apply(URLSearchParams, [void 0, _b.sent()]))();
                        this.__tempOAuthTokenSecret = params.get("oauth_token_secret");
                        return [2 /*return*/, { token: params.get("oauth_token") }];
                }
            });
        });
    };
    OAuth.prototype.onAccessToken = function (token, done) {
        this.storeToken(token.access_token, token.access_token_secret);
        if (this.__onSuccessResolveFunc)
            this.__onSuccessResolveFunc(this.getToken(), done);
        this.__onSuccessResolveFunc = null;
        this.__tempOAuthTokenSecret = null;
    };
    OAuth.prototype.storeKeyFor = function (kind) {
        return this.provider.name + "_" + this.consumer.app_name + "_" + kind + "_" + (this.consumer.scope || "write");
    };
    OAuth.prototype.storeToken = function (access_token, access_token_secret) {
        this.store.setItem(this.storeKeyFor("access_token"), access_token);
        this.store.setItem(this.storeKeyFor("access_token_secret"), access_token_secret);
    };
    OAuth.prototype.getToken = function () {
        return {
            access_token: this.store.getItem(this.storeKeyFor("access_token")),
            access_token_secret: this.store.getItem(this.storeKeyFor("access_token_secret")),
        };
    };
    OAuth.prototype.hasToken = function () {
        var token = this.getToken();
        return !!(token.access_token && token.access_token_secret);
    };
    OAuth.prototype.clearToken = function () {
        this.store.removeItem(this.storeKeyFor("access_token"));
        this.store.removeItem(this.storeKeyFor("access_token_secret"));
    };
    /**
     * Send signed request by the same interface with window.fetch.
     * @param {RequestInfo} input
     * @param {RequestInit} init
     */
    OAuth.prototype.fetch = function (input, init) {
        var token = this.getToken();
        var url = new URL(input.url || input);
        var parameters = {};
        url.searchParams.forEach(function (value, key) {
            parameters[key] = value;
            url.searchParams.delete(key);
        });
        var request = OAuthSimple().sign({
            action: (init || {}).method || "GET",
            path: url.toString(),
            parameters: parameters,
            signatures: {
                consumer_key: this.consumer.consumer_key,
                shared_secret: this.consumer.consumer_secret,
                oauth_token: token.access_token,
                oauth_secret: token.access_token_secret,
            }
        });
        return fetch(request.signed_url, init);
    };
    OAuth.prototype.sign = function () {
        var _a = this.getToken(), access_token = _a.access_token, access_token_secret = _a.access_token_secret;
        var request = OAuthSimple().sign({
            signatures: {
                consumer_key: this.consumer.consumer_key,
                shared_secret: this.consumer.consumer_secret,
                oauth_token: access_token,
                oauth_secret: access_token_secret,
            }
        });
        return request;
    };
    return OAuth;
}());
export { OAuth };
export var SupportedProvider;
(function (SupportedProvider) {
    SupportedProvider["Twitter"] = "twitter";
})(SupportedProvider || (SupportedProvider = {}));
var ProxyPage = /** @class */ (function () {
    function ProxyPage(href /* , private scope = window */) {
        if (href === void 0) { href = window.location.href; }
        this.url = new URL(href);
        var provider = this.url.searchParams.get("provider");
        this.oauth = ProxyPage.getOAuthObjectFromBackground(provider);
    }
    ProxyPage.open = function (url, provider) {
        var u = new URL(url);
        u.searchParams.set("provider", provider.toLowerCase());
        var oauth = this.getOAuthObjectFromBackground(provider);
        return new Promise(function (resolve) {
            oauth.__onSuccessResolveFunc = function (token, done) {
                if (typeof done == "function")
                    done();
                setTimeout(function () { return resolve(token); });
            };
            window.open(u.toString());
        });
    };
    ProxyPage.getOAuthObjectFromBackground = function (provider) {
        return this.extension.getBackgroundPage()[provider];
    };
    ProxyPage.prototype.onload = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                if (!this.oauth) {
                    window.alert("URL parameter `provider` is not set.");
                    return [2 /*return*/, window.close()];
                }
                if (this.oauth.hasToken()) {
                    window.alert("You already have access token.");
                    return [2 /*return*/, this.oauth.onAccessToken(this.oauth.getToken(), function () { return window.close(); })];
                }
                if (this.url.searchParams.get(ProxyPage.callbackIdentifier) == "true") {
                    return [2 /*return*/, this.handleRequestToken()];
                }
                return [2 /*return*/, this.redirectToProvider()];
            });
        });
    };
    ProxyPage.prototype.redirectToProvider = function () {
        return __awaiter(this, void 0, void 0, function () {
            var callback_url, authorize;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        callback_url = new URL(window.location.href);
                        callback_url.searchParams.set(ProxyPage.callbackIdentifier, "true");
                        callback_url.searchParams.set("provider", this.url.searchParams.get("provider"));
                        return [4 /*yield*/, this.oauth.getProviderAuthorizeURL(callback_url.toString())];
                    case 1:
                        authorize = _a.sent();
                        // Jump to the provider's login page
                        // Good luck!! 🚀
                        window.location.replace(authorize.toString());
                        return [2 /*return*/];
                }
            });
        });
    };
    ProxyPage.prototype.handleRequestToken = function () {
        return __awaiter(this, void 0, void 0, function () {
            var token, verifier, request, res, info, _a;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        token = this.url.searchParams.get("oauth_token");
                        verifier = this.url.searchParams.get("oauth_verifier");
                        request = OAuthSimple().sign({
                            path: this.oauth.provider.access_url,
                            parameters: {
                                "oauth_token": token,
                                "oauth_verifier": verifier
                            },
                            signatures: {
                                consumer_key: this.oauth.consumer.consumer_key,
                                shared_secret: this.oauth.consumer.consumer_secret,
                                oauth_secret: this.oauth.__tempOAuthTokenSecret,
                            }
                        });
                        return [4 /*yield*/, fetch(request.signed_url, { method: "GET" })];
                    case 1:
                        res = _b.sent();
                        _a = URLSearchParams.bind;
                        return [4 /*yield*/, res.text()];
                    case 2:
                        info = new (_a.apply(URLSearchParams, [void 0, _b.sent()]))();
                        this.oauth.onAccessToken({
                            access_token: info.get("oauth_token"),
                            access_token_secret: info.get("oauth_token_secret"),
                        }, function () { return window.close(); });
                        return [2 /*return*/];
                }
            });
        });
    };
    ProxyPage.callbackIdentifier = "chromeexoauthcallback";
    ProxyPage.extension = chrome.extension;
    return ProxyPage;
}());
export { ProxyPage };
